@@ -271,7 +271,9 @@ meta_session_init (const char *previous_client_id,
   
   if (session_connection == NULL)
     {
-      meta_warning (_("Failed to a open connection to a session manager, so window positions will not be saved: %s\n"), buf);
+      meta_topic (META_DEBUG_SM, 
+                  "Failed to a open connection to a session manager, so window positions will not be saved: %s\n",
+                  buf);
 
       goto out;
     }
@@ -1008,7 +1010,7 @@ save_state (void)
                 while (w != NULL)
                   {
                     int n;
-                    n = meta_workspace_screen_index (w->data);
+                    n = meta_workspace_index (w->data);
                     fprintf (outfile,
                              "    <workspace index=\"%d\"/>\n", n);
 
@@ -1815,6 +1817,20 @@ io_from_warning_dialog (GIOChannel   *channel,
       /* Remove the callback, freeing data */
       return FALSE; 
     }
+  else if (condition & G_IO_IN)
+    {
+      /* Check for EOF */
+      
+      char buf[16];
+      int ret;
+ 
+      ret = read (d->child_pipe, buf, sizeof (buf));
+      if (ret == 0)
+ 	{
+ 	  finish_interact (d->shutdown);
+ 	  return FALSE;
+ 	}
+    }
 
   /* Keep callback installed */
   return TRUE;
